/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.filter.odf;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.office.filter.api.OCKey;
import com.openexchange.office.filter.core.DLList;
import com.openexchange.office.filter.core.DLNode;
import com.openexchange.office.filter.odt.dom.Annotation;

public class Authors {

    int count = 0;
    final private DLList<Author> authors = new DLList<Author>();

    public void  addAuthor(Annotation annotation) {
        final String a = annotation.getCreator()==null ? "" : annotation.getCreator();
        final String i = annotation.getInitials() == null ? getInitialsFromName(a) : annotation.getInitials();

        final Author author = new Author(a, i);
        DLNode<Author> node = authors.getFirstNode();

        int authorsId = 1;
        while(node!=null) {
            if(node.getData().equals(author)) {
                break;
            }
            node = node.getNext();
            authorsId++;
        }
        if(node==null) {
            authors.add(author);
            count++;
        }
        annotation.setAuthorId(authorsId);
    }

    private String getInitialsFromName(String n) {
        if(n.isEmpty()) {
            return "";
        }
        return n.substring(0, 0);
    }

    public JSONArray getJSON()
        throws JSONException {

        if(authors.isEmpty()) {
            return null;
        }
        final JSONArray json = new JSONArray(count);
        DLNode<Author> node = authors.getFirstNode();
        long id = 1;
        while(node!=null) {
            final JSONObject authorEntry = new JSONObject(3);
            final Author author = node.getData();
            authorEntry.put(OCKey.AUTHOR_ID.value(), id);
            authorEntry.put(OCKey.NAME.value(), author.getName());
            authorEntry.put(OCKey.INITIALS.value(), author.getInitials());
            json.put(authorEntry);
            node = node.getNext();
            id++;
        }
        return json;
    }
}
