/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.tools.doc;

import com.openexchange.exception.OXException;
import com.openexchange.office.tools.error.ErrorCode;

/**
 * Encapsulates the reason of a failure to process a document
 * specific request.
 *
 * {@link OXDocumentException}
 *
 * @author <a href="mailto:carsten.driesner@open-xchange.com">Carsten Driesner</a>
 * @since v7.6.2
 */
public class OXDocumentException extends OXException {

    private static final long serialVersionUID = 1L;

    final private ErrorCode errorCode;

    public OXDocumentException(String message, ErrorCode eCode) {
        super(eCode.getCode(), message);
        errorCode = eCode;
    }

    public OXDocumentException(String message, Throwable e, ErrorCode eCode) {
        super(eCode.getCode(), message, e);
        setStackTrace(e.getStackTrace());
        errorCode = eCode;
    }

    public OXDocumentException(Throwable e, ErrorCode eCode) {
        super(e);
        setStackTrace(e.getStackTrace());
        errorCode = eCode;
    }

    public OXDocumentException(ErrorCode eCode) {
        super(eCode.getCode(), "");
        errorCode = eCode;
    }

    /**
     * Provides the error code which describes the reason of the
     * document processing failure.
     *
     * @return
     *  The error code which describes the reason of the failure.
     */
    public ErrorCode getErrorcode() {
        return errorCode;
    }

    @Override
    public String getMessage() {
        return getLogMessage();
    }

    @Override
    public String toString() {
        return getLogMessage();
    }

    /**
     * Gets the composed log message for this OXDocumentException
     * instance.
     *
     * @return
     *  the composed log message
     */
    public String getLogMessage() {
        final StringBuilder sb = new StringBuilder(256);

        // make sure that our error code is also logged
        sb.append("ErrorCode='");
        sb.append(errorCode.toString());
        sb.append("' ");

        sb.append(super.getLogMessage());

        return sb.toString();
    }
}
